// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file ElementFunction.h */

#ifndef AMDIS_ELEMENTFUNCTION_H
#define AMDIS_ELEMENTFUNCTION_H

#include "AbstractFunction.h"
#include "DOFVector.h"
#include "BasisFunction.h"

namespace AMDiS {

  /// Abstract access to functions living on elements.
  template<typename T>
  class ElementFunction : public AbstractFunction<T, DimVec<double> >
  {
  public:
    /// constructor.
    ElementFunction() 
      : elInfo(NULL) 
    {}

    /// destructor.
    virtual ~ElementFunction() {}

    /// sets \ref elInfo_;
    inline void setElInfo(const ElInfo *elInfo_) 
    { 
      elInfo = elInfo_; 
    }

  protected:
    /// ElInfo the function currently lives on.
    const ElInfo *elInfo;
  };

  /// ElementFunction wich encapsulates the evaluation of an analytical function.
  template<typename T>
  class ElementFunctionAnalytic : public ElementFunction<T>
  {
  public:
    /// constructor
    ElementFunctionAnalytic(const AbstractFunction<T, WorldVector<double> > *fct_)
      : ElementFunction<T>(),
	fct(fct_)
    {}

    /// evaluation at given coordinates.
    T operator()(const DimVec<double>& bary) const 
    {
      WorldVector<double> worldCoords;
      this->elInfo->coordToWorld(bary, worldCoords);
      return (*fct)(worldCoords);
    }

  protected:
    /// function to be avaluated at world coordinates.
    const AbstractFunction<T, WorldVector<double> > *fct;
  };


  /// ElementFunction wich encapsulates the interpolation of an DOFVector.
  template<typename T>
  class ElementFunctionDOFVec : public ElementFunction<T>
  {
  public:
    /// constructor.
    ElementFunctionDOFVec(const DOFVector<T> *vec) 
      : ElementFunction<T>(),
	dofVector(vec)
    {}

    /// evaluation at given coordinates.
    T operator()(const DimVec<double>& bary) const 
    {
      mtl::dense_vector<T> localVec(dofVector->getFeSpace()->getBasisFcts()->getNumber());
      dofVector->getLocalVector(this->elInfo->getElement(), localVec);
      
      return dofVector->getFeSpace()->getBasisFcts()->evalUh(bary, localVec);
    }

  protected:
    /// DOFVector to be interpolated.
    const DOFVector<T> *dofVector;
  };

}

#endif
